/*
 * FCKeditor - The text editor for Internet - http://www.fckeditor.net
 * Copyright (C) 2003-2008 Frederico Caldeira Knabben
 * 
 * == BEGIN LICENSE ==
 * 
 * Licensed under the terms of any of the following licenses at your
 * choice:
 * 
 *  - GNU General Public License Version 2 or later (the "GPL")
 *    http://www.gnu.org/licenses/gpl.html
 * 
 *  - GNU Lesser General Public License Version 2.1 or later (the "LGPL")
 *    http://www.gnu.org/licenses/lgpl.html
 * 
 *  - Mozilla Public License Version 1.1 or later (the "MPL")
 *    http://www.mozilla.org/MPL/MPL-1.1.html
 * 
 * == END LICENSE ==
 */
package net.fckeditor.handlers;


import net.fckeditor.connector.Connector;
import net.fckeditor.requestcycle.UserAction;
import net.fckeditor.requestcycle.UserPathBuilder;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Handler for classes which implement the interfaces from package
 * {@link net.fckeditor.requestcycle}.
 * 
 * @version $Id: RequestCycleHandler.java 2461 2008-09-14 13:41:43Z mosipov $
 */
public class RequestCycleHandler {
	private static Logger logger = LoggerFactory.getLogger(RequestCycleHandler.class);
	private static UserAction userAction = null;
	private static UserPathBuilder userPathBuilder = null;
	private static Connector connector = null;

	static {
		// If there are more objects to instantiate in future, we could solve the following by reflection!
				
		// 1. try to instantiate the UserAction object
		String fqcn = PropertiesLoader.getProperty("connector.userActionImpl");
		if (fqcn == null)
			logger.warn("No property found for UserAction implementation, any user action will be disabled!");
		else {
			try {
				Class<?> clazz = Class.forName(fqcn);
				userAction = (UserAction) clazz.newInstance();
				logger.info("UserAction initialized to {}", userAction.getClass());
			} catch (Exception e) {
				logger.error("Couldn't instantiate class [".concat(fqcn).concat(
				        "], any user action will disabled!"), e);
			}
		}

		// 2. try to instantiate the UserPathBuilder object
		fqcn = PropertiesLoader.getProperty("connector.userPathBuilderImpl");
		if (fqcn == null)
			logger.warn("No property found for UserPathBuilder implementation! "
					.concat("The 'DefaultUserFilesPath' will be used in the ConnectorServlet!"));
		else {
			try {
				Class<?> clazz = Class.forName(fqcn);
				userPathBuilder = (UserPathBuilder) clazz.newInstance();
				logger.info("UserPathBuilder initialized to {}", userPathBuilder.getClass());
			} catch (Exception e) {
				logger.error("Couldn't instantiate class [".concat(fqcn)
				        .concat("], The 'DefaultUserFilesPath' will be used in the ConnectorServlet!"), e);
			}
		}
		
		// TODO Connector should be handled in ConnectorHandler which makes much more sense
		// 3. try to instantiate the Connector object
		fqcn = PropertiesLoader.getProperty("connector.implementation");
		if (fqcn == null)
			logger.warn("No property found for Connector implementation, any user action will be disabled!");
		else {
			try {
				Class<?> clazz = Class.forName(fqcn);
				connector = (Connector) clazz.newInstance();
				logger.info("Connector initialized to {}", connector.getClass());
			} catch (Exception e) {
				logger.error("Couldn't instantiate class [".concat(fqcn).concat(
				        "], any user action will disabled!"), e);
			}
		}

	}

	/**
	 * Just a wrapper to
	 * {@link UserAction#isEnabledForFileBrowsing()}.
	 * 
	 * @return {@link UserAction#isEnabledForFileBrowsing()}
	 *         or false if {@link UserAction} isn't set.
	 */
	public static boolean isEnabledForFileBrowsing() {
		return (userAction != null && userAction.isEnabledForFileBrowsing());
	}

	/**
	 * Just a wrapper to
	 * {@link UserAction#isEnabledForFileUpload()}.
	 * 
	 * @return {@link UserAction#isEnabledForFileUpload()} or
	 *         false if {@link UserAction} isn't set.
	 */
	public static boolean isEnabledForFileUpload() {
		return (userAction != null && userAction.isEnabledForFileUpload());
	}

	/**
	 * Just a wrapper to
	 * {@link UserAction#isEnabledForFolderCreation()}.
	 * 
	 * @return {@link UserAction#isEnabledForFolderCreation()} or
	 *         false if {@link UserAction} isn't set.
	 */
	public static boolean isEnabledForFolderCreation() {
		return (userAction != null && userAction.isEnabledForFolderCreation());
	}

	/**
	 * Getter for the <code>UserFilesPath</code>.<br />
	 * Method is used by other handlers only!
	 * 
	 * @param request
	 * @return {@link UserPathBuilder#getUserFilesPath()} or
	 *         <code>null</code> if <code>userPathBuilder</code> is
	 *         <code>null</code>.
	 */
	public static String getUserFilePath() {
		return (userPathBuilder != null) ? userPathBuilder.getUserFilesPath() : null;
	}
	
	/**
	 * Getter for the implementation of {@link Connector}.
	 * 
	 * @return Implementation of {@link Connector}.
	 */
	public static Connector getConnector() {
		return connector;
	}
}
